@@ -2,7 +2,6 @@
 # Author: Óscar Nájera
 # License: 3-clause BSD
 """
-==================
 RST file generator
 ==================
 
@@ -51,31 +50,26 @@ except ImportError:
 
 from io import StringIO
 
-try:
-    # make sure that the Agg backend is set before importing any
-    # matplotlib
-    import matplotlib
-    matplotlib.use('agg')
-    matplotlib_backend = matplotlib.get_backend()
+# make sure that the Agg backend is set before importing any
+# matplotlib
+import matplotlib
+matplotlib.use('agg')
+matplotlib_backend = matplotlib.get_backend()
 
-    if matplotlib_backend != 'agg':
-        mpl_backend_msg = (
-            "Sphinx-Gallery relies on the matplotlib 'agg' backend to "
-            "render figures and write them to files. You are "
-            "currently using the {} backend. Sphinx-Gallery will "
-            "terminate the build now, because changing backends is "
-            "not well supported by matplotlib. We advise you to move "
-            "sphinx_gallery imports before any matplotlib-dependent "
-            "import. Moving sphinx_gallery imports at the top of "
-            "your conf.py file should fix this issue")
+if matplotlib_backend != 'agg':
+    mpl_backend_msg = (
+        "Sphinx-Gallery relies on the matplotlib 'agg' backend to "
+        "render figures and write them to files. You are "
+        "currently using the {} backend. Sphinx-Gallery will "
+        "terminate the build now, because changing backends is "
+        "not well supported by matplotlib. We advise you to move "
+        "sphinx_gallery imports before any matplotlib-dependent "
+        "import. Moving sphinx_gallery imports at the top of "
+        "your conf.py file should fix this issue")
 
-        raise ValueError(mpl_backend_msg.format(matplotlib_backend))
+    raise ValueError(mpl_backend_msg.format(matplotlib_backend))
 
-    import matplotlib.pyplot as plt
-except ImportError:
-    # this script can be imported by nosetest to find tests to run: we should
-    # not impose the matplotlib requirement in that case.
-    pass
+import matplotlib.pyplot as plt
 
 from . import glr_path_static
 from .backreferences import write_backreferences, _thumbnail_div
@@ -305,16 +299,17 @@ def figure_rst(figure_list, sources_dir):
         number of figures saved
     """
 
-    figure_list = [os.path.relpath(figure_path, sources_dir)
-                   for figure_path in figure_list]
+    figure_paths = [os.path.relpath(figure_path, sources_dir)
+                    .replace(os.sep, '/').lstrip('/')
+                    for figure_path in figure_list]
     images_rst = ""
-    if len(figure_list) == 1:
-        figure_name = figure_list[0]
-        images_rst = SINGLE_IMAGE % figure_name.lstrip('/')
-    elif len(figure_list) > 1:
+    if len(figure_paths) == 1:
+        figure_name = figure_paths[0]
+        images_rst = SINGLE_IMAGE % figure_name
+    elif len(figure_paths) > 1:
         images_rst = HLIST_HEADER
-        for figure_name in figure_list:
-            images_rst += HLIST_IMAGE_TEMPLATE % figure_name.lstrip('/')
+        for figure_name in figure_paths:
+            images_rst += HLIST_IMAGE_TEMPLATE % figure_name
 
     return images_rst, len(figure_list)
 
@@ -422,16 +417,18 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
         computation_times.append((time_elapsed, fname))
         new_fname = os.path.join(src_dir, fname)
         intro = extract_intro(new_fname)
-        write_backreferences(seen_backrefs, gallery_conf,
-                             target_dir, fname, intro)
         this_entry = _thumbnail_div(build_target_dir, fname, intro) + """
 
 .. toctree::
    :hidden:
 
-   /%s/%s\n""" % (build_target_dir, fname[:-3])
+   /%s\n""" % os.path.join(build_target_dir, fname[:-3]).replace(os.sep, '/')
         entries_text.append((amount_of_code, this_entry))
 
+        if gallery_conf['backreferences_dir']:
+            write_backreferences(seen_backrefs, gallery_conf,
+                                 target_dir, fname, intro)
+
     # sort to have the smallest entries in the beginning
     entries_text.sort()
 
